/*
  作成者: ihara789
  作成日: 2023/12/01
  更新日: 2023/12/14
*/

/* ------------- */

// ------------------ 単語リスト
const words = [
  "getElementById",
  "getElementsByClassName",
  "getElementsByTagName",
  "querySelector",
  "querySelectorAll",
  "innerText",
  "setAttribute",
  "removeAttribute",
  "createElement",
  "appendChild",
  "removeChild",
  "addEventListener",
  "removeEventListener"
];

// ------------------ 並び替え

function shuffle_words(shuffle_order) {
  for (let i = shuffle_order.length - 1; i > 0; i--) {
    const j = Math.floor(Math.random() * (i + 1));
    const temp = shuffle_order[i];
    shuffle_order[i] = shuffle_order[j];
    shuffle_order[j] = temp;
  }
  return shuffle_order;
}
const shuffle_order = words;
const shuffled_words = shuffle_words(shuffle_order);

//------------------ 初期状態

const questionElement = document.getElementById("question")
const clearedElement = document.getElementById("cleared")
const inputElement = document.getElementById("input")
const scoreElement = document.getElementById("score");
const resetBtn = document.getElementById("reset-btn");

let score = 0;                    // スコア
let isPlaying = false;            // 開始
let isClear = false;              // 終了
let isTimeup = false              // 時間切れ
let question = shuffled_words;    // 問題
// console.log(question);
let question_count = question.length - 1; // 問題数
let answer_count = 0;             // 解答数
let question_length = question[answer_count].length; // 出題文字数
let answer_length = 0;            // 正解文字数
let correct_word                  // 正解文字
let key_code                      // 入力文字
let key_code_formatted           // 入力文字整形
let input_text                    // 入力表示

//------------------ 入力処理

window.addEventListener("keydown", push_Keydown);

function push_Keydown(event) {
  key_code = event.key;

  // 正解文字準備
  correct_word = question[answer_count].charAt(answer_length);

  // 入力文字整形（文字以外の表示崩れ対策）
  if (/^[a-zA-Z0-9]$/.test(event.key)) {
    key_code_formatted = key_code
  } else {
    key_code_formatted = "※"
  }

  if (!isPlaying) {
    // 開始
    questionElement.innerHTML = question[answer_count];
    clearedElement.style.visibility = "hidden";
    inputElement.innerHTML = "タイピング開始：文字を入力してください";
    isPlaying = true;
    // console.log("開始");

  } else if ( isTimeup == true && question_count !== answer_count){
    questionElement.innerHTML = "タイムアップ";
    clearedElement.innerHTML = "タイムアップ";
    clearedElement.style.visibility = "hidden";
    isClear = true;
    // console.log("TimeUp");

  } else {

    if (key_code !== correct_word) {

      // 不正解
      input_text = "入力：<strong>" + key_code_formatted + "</strong>";
      if (correct_word.length === 0) {
        // console.log("終了");
      } else {
        input_text = input_text + " <span>ヒント：次の文字は<strong> " + correct_word + " </strong>です</span>";
        inputElement.innerHTML = input_text;
        // console.log("NG");
      }

    } else {

      // 正解
      answer_length = answer_length + 1;
      correct_word = question[answer_count].charAt(answer_length);
      score++;

      clearedElement.innerHTML = question[answer_count].slice(0, answer_length);
      clearedElement.style.visibility = "visible";
      inputElement.innerHTML = "入力：<strong> " + key_code_formatted + "</strong> ";
      scoreElement.textContent = `Score: ${score}`;
      // console.log("OK");

      if (question_length <= answer_length) {
        // 全文字正解

        if (question_count == answer_count) {
          // 全問正解

          questionElement.innerHTML = "終了";
          clearedElement.innerHTML = "終了";
          clearedElement.style.visibility = "hidden";
          isClear = true;
          // console.log("End");

        } else {

          // 次の問題
          answer_count = answer_count + 1;
          answer_length = 0;
          correct_word = question[answer_count].charAt(answer_length);
          question_length = question[answer_count].length;
          questionElement.innerHTML = question[answer_count];
          clearedElement.style.visibility = "hidden";

          // console.log("Next");
        }
      }
    }
  }
  // console.log("問題数" + answer_count + "/" + question_count);
  // console.log("出題文字数" + question_length);
  // console.log("正解文字数" + answer_length);
  // console.log("正解文字" + correct_word);
}

//------------------ リセットボタン

function reload() {
  location.reload();
}
resetBtn.addEventListener("click", () => {
  window.location.reload();
});

//------------------ カウントダウン

let count = 30;
let intervalId;
let isCountingDown = false;
document.addEventListener("keydown", function (event) {
  if (!isCountingDown) {
    isCountingDown = true;
    intervalId = setInterval(() => {
      document.getElementById("time").textContent = "Time: " + count;
      if (count <= 0 || isClear == true) {
        clearInterval(intervalId);
        isTimeup = true;
      } else {
        count--;
      }
    }, 1000);
  }
});
